/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: button.h,v 1.11 2001/02/03 17:00:36 malekith Exp $ */

#ifndef _DONT_INCLUDE_DIRECTLY
#error "button.h is intended to be included from <v/v.h>"
#endif

class Button : public Widget {
public:
	virtual void draw();
	virtual void handle_event(Event &e);
	virtual bool is_accel(Event &e);
	
	// new:
	String caption;
	void compute_sizes();
	void set_caption(const String &r)
		{ caption = r; compute_sizes(); }
	virtual void execute() { }
	Button *init(int x, int y, const String &c, int w = 0)
		{ pos.move_to(x,y); set_caption(c); if (w) pos.w = w; 
		  return this; }
	Button() { name = "Button"; }
};

class CommandButton : public Button {
public:
	int command;
	CommandButton() : command(ev_none) { name = "CommandButton"; }
	virtual void execute()
	{
		Event e;
		e.type = Event::command;
		e.data = command;
		parent->handle_event(e);
	}
	CommandButton *init(int x, int y, const String &c, int cmd, int w = 0)
		{ Button::init(x,y,c,w); command = cmd; return this; }
};

class Label : public Widget {
public:
	virtual void draw();
	
	// new:
	int fcolor;
	enum Align {
		left,
		right,
		center,
	} align;
	String caption;
	void compute_sizes();
	void set_caption(const String &r)
		{ caption = r; compute_sizes(); }
	virtual bool can_get_focus() { return false; }
	Label() : fcolor(color("label")), align(left) { name = "Label";  }
	Label(int x, int y, const String &c, int col = 0, Align al = left,
	      int w = 0) :
			fcolor(col ? col : color("label")), align(al)
	{
		pos.move_to(x,y); set_caption(c); 
		if (w) pos.w = w;
		name = "Label";
	}
};

class InputLine : public Widget {
public:
	int cp, top;
	bool deadly, ghosted, fixed;
	String caption;
	int limit;

	virtual void activate_hook();
	virtual void draw();
	virtual void handle_event(Event &e);
	virtual bool can_get_focus() { return !ghosted; }
	InputLine(int x = 0, int y = 0, int w = 0, const String &c = "") :
		cp(0), top(0), deadly(false), ghosted(false), fixed(true),
		caption(c), limit(0)
		{ name = "InputLine"; pos.set(x,y,w,1); }
};
