/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: listbox.cc,v 1.8 2000/10/22 21:03:32 malekith Exp $ */

#define _V_PRIVATE
#include <v/v.h>
#include <v/term.h>
#include <v/listbox.h>


void ListItem::draw_at(int x, int y, int w, int col, int col2, ListBox *,
		       int offset)
{
	go(x, y);
	if (hl)
		v_alt_print((int)strlen(caption) > offset ? 
				caption.cstr() + offset : "", col, col2, w);
	else {
		set_color(col);
		print("%-*.*s", w, w, (int)strlen(caption) > offset ? 
						caption.cstr() + offset : "");
	}
}

void ListBox::draw()
{
	int x = pos_x(), y = pos_y();
	int k = pos.h;
	int cs = color("listitem_selected"), 
	    cn = color("listitem_normal");
	int csh = color("listitem_hl_selected"), 
	    cnh = color("listitem_hl_normal");
	
	if (active_item >= items.size())
		active_item = items.size() - 1;
	if (active_item < 0)
		active_item = 0;
		
	if (top_item > items.size() - pos.h)
		top_item = items.size() - pos.h;
	if (top_item < 0)
		top_item = 0;
	
	if (active_item < top_item)
		top_item = active_item;
	if (active_item >= top_item + pos.h)
		top_item = active_item - pos.h + 1;

	for (int i=top_item; i < items.size() && k-- > 0; i++)
		items[i]->draw_at(x, y++, pos.w, 
				  i == active_item ? cs : cn,
				  i == active_item ? csh : cnh, this,
				  offset);
	set_color(color("listitem_normal"));
	while (k-- > 0) {
		go(x,y++);
		print("%*s",pos.w,"");
	}
	go(x + pos.w - 13, y);
	print(" Linia %d ", active_item + 1);
}

void ListBox::handle_event(Event &e)
{
	Widget::handle_event(e);
	if (e.type != Event::keyboard)
		return;
		
	switch (e.data) {
	case key_left:
		if (offset)
			offset--;
		break;
	case key_right:
		offset++;
		break;
	case key_up:
	case '-':
		active_item--;
		break;
	case key_down:
	case '=':
	case '+':
		active_item++;
		break;
	case '\n':
		if (items.size() > active_item)
			items[active_item]->execute(this);
		break;
	case key_pgup:
		active_item -= pos.h;
		top_item -= pos.h;
		if (active_item > 0)
			break;
	case key_home:
		active_item = 0;
		break;
	case key_pgdn:
		active_item += pos.h;
		top_item += pos.h;
		if (active_item < items.size())
			break;
	case key_end:
		active_item = items.size() - 1;
		break;
	default:
		return;
	}
	e.kill();
	
	draw();
}

void ListBox::free()
{
	while (items.size())
		delete items.pop();
}

ListBox::~ListBox()
{
	free();
}
